/*!
 * @file        apm32f00x_flash.h
 *
 * @brief       This file contains all the functions prototypes,enumeration and macro for the FLASH firmware library      
 *
 * @version     V1.0.0
 *
 * @date        2019-10-15
 *
 */

#ifndef __APM32F00X_FLASH_H
#define __APM32F00X_FLASH_H

#ifdef __cplusplus
 extern "C" {
#endif

#include "apm32f00x.h"	 

/** @addtogroup Peripherals_Library Standard Peripheral Library  
  @{
*/

/** @addtogroup FLASH_Driver FLASH Driver
  @{
*/

/** @addtogroup FLASH_Enumerations Enumerations
  @{
*/

/**
 * @brief   FLASH Status
 */
typedef enum
{
    FLASH_STATE_COMPLETE    = ((uint8_t)0), //!< Operation complete
    FLASH_STATE_BUSY        = ((uint8_t)1), //!< Busy
    FLASH_STATE_PG_ERR      = ((uint8_t)2), //!< Program error
    FLASH_STATE_WRP_ERR     = ((uint8_t)3), //!< Write Protection error
    FLASH_STATE_TIMEOUT     = ((uint8_t)4), //!< Time out
}FLASH_STATE_T;

/**
 * @brief   Flash latency
 */
typedef enum
{
    FLASH_LATENCY_0         = ((uint8_t)0),
    FLASH_LATENCY_1         = ((uint8_t)1),
    FLASH_LATENCY_2         = ((uint8_t)2),
}FLASH_LATENCY_T;

/**
 * @brief   Option byte Window watch dog activation
 */
typedef enum
{
    FLASH_OB_WWDT_HW        = ((uint16_t)0X00),     //!< activated by hardware
    FLASH_OB_WWDT_SW        = ((uint16_t)0X01),     //!< activated by software
}FLASH_OB_WWDT_T;

/**
 * @brief   Window watch dog halt reset
 */
typedef enum
{
    FLASH_OB_WWDT_HALT_RST      = ((uint16_t)0x00), //!< Reset generate on halt if WWDT active
    FLASH_OB_WWDT_HALT_NORST    = ((uint16_t)0x02), //!< No reset generate on halt if WWDT active
}FLASH_OB_WWDT_HALT_T;

/**
 * @brief Option byte Independent watch dog activation
 */
typedef enum
{
    FLASH_OB_IWDT_HW        = ((uint16_t)0X00),     //!< activated by hardware
    FLASH_OB_IWDT_SW        = ((uint16_t)0X04),     //!< activated by software
}FLASH_OB_IWDT_T;

/**
 * @brief   LIRC clock Enable or Disable
 */
typedef enum
{
    FLASH_OB_LIRCEN_ENABLE  = ((uint16_t)0X00),     //!< LIRC clock is available as cpu clock source
    FLASH_OB_LIRCEN_DISABLE = ((uint16_t)0X08),     //!< LIRC clock is not available as cpu clock source
}FLASH_OB_LIRCEN_T;

/**
 * @brief   HIRC trimming
 */
typedef enum
{
    FLASH_OB_HIRC_TRIM_4    = ((uint16_t)0x00),      //!< 4-bit trimming supported  
    FLASH_OB_HIRC_TRIM_3    = ((uint16_t)0x10),      //!< 3-bit trimming supported
}FLASH_OB_HIRC_TRIM_T;

/**
 * @brief   Alternate function remap state 
 */
typedef enum
{
    FLASH_OB_AFR_DISABLE    = ((uint16_t)0X00),     //!< Disable Alternate function remap
    FLASH_OB_AFR_ENABLE     = ((uint16_t)0X01),     //!< Enable Alternate function remap
}FLASH_OB_AFR_T;

/**
 * @brief   Wakup timer divider
 */
typedef enum
{
    FLASH_OB_WUPT_DIV_4M_128K   = ((uint16_t)0x00), //!< 4 MHz to 128 kHz prescaler
    FLASH_OB_WUPT_DIV_8M_128K   = ((uint16_t)0x01), //!< 8 MHz to 128 kHz prescaler
    FLASH_OB_WUPT_DIV_16M_128K  = ((uint16_t)0x02), //!< 8 MHz to 128 kHz prescaler
}FLASH_OB_WUPT_DIV_T;

/**
 * @brief   Wakup timer clock source
 */
typedef enum
{
    FLASH_OB_WUPT_CLKSRC_HXT_DIV    = ((uint16_t)0x00),     //!< HXT divider as Wakeup timer clock source 
    FLASH_OB_WUPT_CLKSRC_LIRC       = ((uint16_t)0x04),     //!< LIRC divider as Wakeup timer clock source
}FLASH_OB_WUPT_CLKSRC_T;

/**
 * @brief   External clock selection
 */
typedef enum
{
    FLASH_OB_EXTCLK_OSCIN       = ((uint16_t)0X00),     //!< External clock signal on OSCIN
    FLASH_OB_EXTCLK_OSCIN_OUT   = ((uint16_t)0X08),     //!< External crystal connected to OSCIN/OSCOUT
}FLASH_OB_EXTCLK_T;

/**
 * @brief   HXT stabilization time
 */
typedef enum
{
    FLASH_OB_HXTCNT_HALF_CYCLE  = ((uint16_t)0x1E),     //!< HXT stabilization time is 0.5 HXT CYCLE 
    FLASH_OB_HXTCNT_8_CYCLE     = ((uint16_t)0x2D),     //!< HXT stabilization time is 8 HXT CYCLE
    FLASH_OB_HXTCNT_128_CYCLE   = ((uint16_t)0x4B),     //!< HXT stabilization time is 128 HXT CYCLE
    FLASH_OB_HXTCNT_2048_CYCLE  = ((uint16_t)0xFF),     //!< HXT stabilization time is 2048 HXT CYCLE
}FLASH_OB_HXTCNT_T;

/**
 * @brief   Interrupt source
 */
typedef enum
{
    FLASH_INT_ERROR             = ((uint32_t)0x400),    //!< Error interrupt
    FLASH_INT_COMPLETE          = ((uint32_t)0x1000),   //!< Operation complete interrupt
}FLASH_INT_T;

/**
 * @brief   Flags definition
 */
typedef enum
{
    FLASH_FLAG_BUSY             = ((uint32_t)0x01),     //!< Busy flag
    FLASH_FLAG_PE               = ((uint32_t)0x04),     //!< Program error flag
    FLASH_FLAG_WPE              = ((uint32_t)0x10),     //!< Write protection flag
    FLASH_FLAG_OC               = ((uint32_t)0x20),     //!< Operation complete flag
    FLASH_FLAG_OBERR            = ((uint32_t)0x0100),   //!< Option byte error flag
}FLASH_FLAG_T;

/**
 * @brief   Read protection state.Enable or Disable
 */
typedef enum
{
    FLASH_RDP_ENABLE        = 0,
    FLASH_RDP_DISABLE       = 0xA5,
}FLASH_RDP_T;

/**@} end of group FLASH_Enumerations*/
    
/** @addtogroup FLASH_Macros Macros
  @{
*/
    
/** Flash write protect page definition */
#define FLASH_WRP_PAGE_0        ((uint32_t)0x00000001)
#define FLASH_WRP_PAGE_1        ((uint32_t)0x00000002)
#define FLASH_WRP_PAGE_2        ((uint32_t)0x00000004)
#define FLASH_WRP_PAGE_3        ((uint32_t)0x00000008)
#define FLASH_WRP_PAGE_4        ((uint32_t)0x00000010)
#define FLASH_WRP_PAGE_5        ((uint32_t)0x00000020)
#define FLASH_WRP_PAGE_6        ((uint32_t)0x00000040)
#define FLASH_WRP_PAGE_7        ((uint32_t)0x00000080)    
#define FLASH_WRP_PAGE_8        ((uint32_t)0x00000100)
#define FLASH_WRP_PAGE_9        ((uint32_t)0x00000200)
#define FLASH_WRP_PAGE_10       ((uint32_t)0x00000400)
#define FLASH_WRP_PAGE_11       ((uint32_t)0x00000800)
#define FLASH_WRP_PAGE_12       ((uint32_t)0x00001000)
#define FLASH_WRP_PAGE_13       ((uint32_t)0x00002000)
#define FLASH_WRP_PAGE_14       ((uint32_t)0x00004000)
#define FLASH_WRP_PAGE_15       ((uint32_t)0x00008000)
#define FLASH_WRP_PAGE_16       ((uint32_t)0x00010000)
#define FLASH_WRP_PAGE_17       ((uint32_t)0x00020000)
#define FLASH_WRP_PAGE_18       ((uint32_t)0x00040000)
#define FLASH_WRP_PAGE_19       ((uint32_t)0x00080000)
#define FLASH_WRP_PAGE_20       ((uint32_t)0x00100000)
#define FLASH_WRP_PAGE_21       ((uint32_t)0x00200000)
#define FLASH_WRP_PAGE_22       ((uint32_t)0x00400000)
#define FLASH_WRP_PAGE_23       ((uint32_t)0x00800000)
#define FLASH_WRP_PAGE_24       ((uint32_t)0x01000000)
#define FLASH_WRP_PAGE_25       ((uint32_t)0x02000000)
#define FLASH_WRP_PAGE_26       ((uint32_t)0x04000000)
#define FLASH_WRP_PAGE_27       ((uint32_t)0x08000000)
#define FLASH_WRP_PAGE_28       ((uint32_t)0x10000000)
#define FLASH_WRP_PAGE_29       ((uint32_t)0x20000000)
#define FLASH_WRP_PAGE_30       ((uint32_t)0x40000000)
#define FLASH_WRP_PAGE_31       ((uint32_t)0x80000000)   
#define FLASH_WRP_PAGE_ALL      ((uint32_t)0xFFFFFFFF)
    
/**@} end of group FLASH_Macros*/

/** @addtogroup FLASH_Structure Data Structure
  @{
*/

/**
 * @brief   User Option byte config struct definition
 */
typedef struct
{
    FLASH_OB_WWDT_T wwdtSw;             //!< Window watch dog activation
    FLASH_OB_IWDT_T iwdtSw;             //!< Independent watch dog activation
    FLASH_OB_WWDT_HALT_T wwdtHalt;      //!< Window watch dog halt reset
    FLASH_OB_LIRCEN_T LIRCen;           //!< Enable or Disable LIRC as CPU clock source
    FLASH_OB_HIRC_TRIM_T trimHIRC;      //!< HIRC timming value
}FLASH_UserConfig_T;

/**
 * @brief   Option byte Alternate function remap config struct definition
 */
typedef struct
{
    FLASH_OB_AFR_T AFR0;                //!< AF0.Can be one of value @ref FLASH_OB_AFR_T
    FLASH_OB_AFR_T AFR1;                //!< AF1.Can be one of value @ref FLASH_OB_AFR_T
    FLASH_OB_AFR_T AFR3;                //!< AF2.Can be one of value @ref FLASH_OB_AFR_T
    FLASH_OB_AFR_T AFR4;                //!< AF3.Can be one of value @ref FLASH_OB_AFR_T
    FLASH_OB_AFR_T AFR7;                //!< AF4.Can be one of value @ref FLASH_OB_AFR_T
}FLASH_AFRConfig_T;

/**@} end of group FLASH_Structure*/

/** @addtogroup FLASH_Fuctions Fuctions
  @{
*/

/** Latency */
void FLASH_SetLatency(FLASH_LATENCY_T latency);

/** Hlaf Cycle Access */
void FLASH_EnableHlafCycleAccess(void);
void FLASH_DisableHlafCycleAccess(void);

/** Prefetch Buffer */
void FLASH_EnablePrefetchBuffer(void);
void FLASH_DisablePrefetchBuffer(void);
uint8_t FLASH_GetPrefetchBufferStatus(void);

/** Lock */
void FLASH_Unlock(void);
void FLASH_Lock(void);

/** State management */
FLASH_STATE_T FLASH_ReadState(void);
FLASH_STATE_T FLASH_WaitForReady(uint32_t timeOut);

/** Erase and Program */
FLASH_STATE_T FLASH_ErasePage(uint32_t pageAddr);
FLASH_STATE_T FLASH_EraseAllPages(void);
FLASH_STATE_T FLASH_EraseOptionByte(void);
FLASH_STATE_T FLASH_ProgramWord(uint32_t addr, uint32_t data);
FLASH_STATE_T FLASH_ProgramHalfWord(uint32_t addr, uint16_t data);
FLASH_STATE_T FLASH_ProgramOptionByte(uint32_t addr, uint16_t data);

/** Write and Read Protection */
FLASH_STATE_T FLASH_EnableWriteProtection(uint32_t page);
FLASH_STATE_T FLASH_ConfigReadOutProtection(FLASH_RDP_T rdp);
uint32_t FLASH_ReadOptionByteWriteProtection(void);
uint8_t FLASH_GetReadProtectionStatus(void);

/** Option Byte Configruration and Read */
FLASH_STATE_T FLASH_ConfigOptionByteUser(FLASH_UserConfig_T *userConfig);
FLASH_STATE_T FLASH_ConfigOptionByteAFR(FLASH_AFRConfig_T *AFRConfig);
FLASH_STATE_T FLASH_ConfigOptionByteClk(FLASH_OB_EXTCLK_T extClk, FLASH_OB_WUPT_CLKSRC_T wuptClk, 
                                                    FLASH_OB_WUPT_DIV_T wuptDiv);
FLASH_STATE_T FLASH_ConfigOptionByteHXTCNT(FLASH_OB_HXTCNT_T counter);
uint32_t FLASH_ReadOptionByteUser(void);

/** Interrupt and Flag */
void FLASH_EnableInterrupt(uint32_t interrupt);
void FLASH_DisableInterrupt(uint32_t interrupt);
uint8_t FLASH_ReadFlag(FLASH_FLAG_T flag);
void FLASH_ClearFlag(FLASH_FLAG_T flag);

/**@} end of group FLASH_Fuctions*/
/**@} end of group FLASH_Driver */
/**@} end of group Peripherals_Library*/

#ifdef __cplusplus
}
#endif

#endif /* __APM32F00X_FLASH_H */

