/*!
 * @file        apm32f00x_flash.c
 *
 * @brief       This file provides all the FLASH firmware functions       
 *
 * @version     V1.0.0
 *
 * @date        2019-10-15
 *
 */

#include "apm32f00x_flash.h"

/** FLASH Read protection key */
#define FLASH_RP_KEY            ((uint32_t)0XA5)

/** Flash key definition */
#define FLASH_KEY_1             ((uint32_t)0x45670123)
#define FLASH_KEY_2             ((uint32_t)0xCDEF89AB)

/** Delay definition */
#define FLASH_DELAY_ERASE       ((uint32_t)0x000B0000)
#define FLASH_DELAY_PROGRAM     ((uint32_t)0x00002000)

/** @addtogroup Peripherals_Library Standard Peripheral Library  
  @{
*/

/** @addtogroup FLASH_Driver FLASH Driver
  @{
*/

/** @addtogroup FLASH_Fuctions Fuctions
  @{
*/

/*!
 * @brief       Set latency
 *
 * @param       latency:    Specifies the FLASH Latency value
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_SetLatency(FLASH_LATENCY_T latency)
{
    FMC->CTRL1_B.LATENCY = latency;
}

/*!
 * @brief       Enable the Half cycle flash access
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_EnableHlafCycleAccess(void)
{
    FMC->CTRL1_B.HCAEN = BIT_SET;
}

/*!
 * @brief       Disable the Half cycle flash access
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_DisableHlafCycleAccess(void)
{
    FMC->CTRL1_B.HCAEN = BIT_RESET;
}

/*!
 * @brief       Enable the Prefetch Buffer
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_EnablePrefetchBuffer(void)
{
    FMC->CTRL1_B.PBEN = BIT_SET;
}

/*!
 * @brief       Disable the Prefetch Buffer
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_DisablePrefetchBuffer(void)
{
    FMC->CTRL1_B.PBEN = BIT_RESET;
}

/*!
 * @brief       Unlocks the FLASH Program Erase Controller
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_Unlock(void)
{
    FMC->KEY = FLASH_KEY_1;
    FMC->KEY = FLASH_KEY_2;
}

/*!
 * @brief       Locks the FLASH Program Erase Controller
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_Lock(void)
{
    FMC->CTRL2_B.LOCK = BIT_SET;
}

/*!
 * @brief       Read flash state
 *
 * @param       None
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_ReadState(void)
{
    uint32_t status;
    FLASH_STATE_T state = FLASH_STATE_COMPLETE;

    status = FMC->STS;
    
    if(status & FLASH_FLAG_PE)
    {
        state = FLASH_STATE_PG_ERR;
    }
    else if(status & FLASH_FLAG_WPE)
    {
        state = FLASH_STATE_WRP_ERR;
    }
    else if(status == FLASH_FLAG_BUSY)
    {
        state = FLASH_STATE_BUSY;
    }

    return state;
}

/*!
 * @brief       Wait for flash controler ready 
 *
 * @param       timeOut:    Specifies the time to wait
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_WaitForReady(uint32_t timeOut)
{
    FLASH_STATE_T state;

    do
    {
        state = FLASH_ReadState();
        timeOut--;
    }while((state == FLASH_STATE_BUSY) && (timeOut));

    if(!timeOut)
    {
        state = FLASH_STATE_TIMEOUT;
    }

    return state;
}

/*!
 * @brief       Erases a specified FLASH page
 *
 * @param       pageAddr:   Specifies the page address
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_ErasePage(uint32_t pageAddr)
{
    FLASH_STATE_T state;

    state = FLASH_WaitForReady(FLASH_DELAY_ERASE);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->CTRL2_B.PAGEERA = BIT_SET;
        
        FMC->ADDR = pageAddr;
        
        FMC->CTRL2_B.STA = BIT_SET;

        state = FLASH_WaitForReady(FLASH_DELAY_ERASE);

        FMC->CTRL2_B.PAGEERA = BIT_RESET;
    }

    return state;
}

/*!
 * @brief       Erases all FLASH pages
 *
 * @param       None
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_EraseAllPages(void)
{
    FLASH_STATE_T state;

    state = FLASH_WaitForReady(FLASH_DELAY_ERASE);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->CTRL2_B.MASSERA = BIT_SET;
        FMC->CTRL2_B.STA = BIT_SET;

        state = FLASH_WaitForReady(FLASH_DELAY_ERASE);

        FMC->CTRL2_B.MASSERA = BIT_RESET;
    }

    return state;
}

/*!
 * @brief       Erase the FLASH option bytes
 *
 * @param       None
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_EraseOptionByte(void)
{
    uint16_t rpKey;
    FLASH_STATE_T state;

    rpKey = FMC->OBCS_B.READPROT ? 0 : FLASH_RP_KEY;

    state = FLASH_WaitForReady(FLASH_DELAY_ERASE);
    
    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->OBKEY = FLASH_KEY_1;
        FMC->OBKEY = FLASH_KEY_2;

        FMC->CTRL2_B.OBE = BIT_SET;
        FMC->CTRL2_B.STA = BIT_SET;

        state = FLASH_WaitForReady(FLASH_DELAY_ERASE);

        if(state == FLASH_STATE_COMPLETE)
        {
            FMC->CTRL2_B.OBE = BIT_RESET;

            FMC->CTRL2_B.OBP = BIT_SET;
            
            OB->READPROT = rpKey;

            state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);
            
            if(state != FLASH_STATE_TIMEOUT)
            {
                FMC->CTRL2_B.OBP = BIT_RESET;
            }
        }
        else if(state != FLASH_STATE_TIMEOUT)
        {
            FMC->CTRL2_B.OBE = BIT_RESET;
        }
    }

    return state;
}

/*!
 * @brief       Program a word at a specified address
 *
 * @param       addr:   Specifies the address to be programmed
 *
 * @param       data:   Specifies the data to be programmed 
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_ProgramWord(uint32_t addr, uint32_t data)
{
    FLASH_STATE_T state;

    state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->CTRL2_B.PG = BIT_SET;

        *(__IO uint16_t *)addr = (uint16_t)data;
        
        state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);
        
        if(state == FLASH_STATE_COMPLETE)
        {
            *(__IO uint16_t *)(addr + 2) = (uint16_t)(data >> 16);
            
            state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);
        }

        FMC->CTRL2_B.PG = BIT_RESET;
    }

    return state;
}

/*!
 * @brief       Programs a half word at a specified address
 *
 * @param       addr:   Specifies the address to be programmed
 *
 * @param       data:   Specifies the data to be programmed
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T    
 *
 * @note       
 */
FLASH_STATE_T FLASH_ProgramHalfWord(uint32_t addr, uint16_t data)
{
    FLASH_STATE_T state;

    state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->CTRL2_B.PG = BIT_SET;

        *(__IO uint16_t*)addr = data;

        state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

        FMC->CTRL2_B.PG = BIT_RESET;
    }

    return state;
}

/*!
 * @brief       Programs a half word at a specified Option Byte Data address
 *
 * @param       addr:   Specifies the address to be programmed
 *
 * @param       data:   Specifies the data to be programmed
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_ProgramOptionByte(uint32_t addr, uint16_t data)
{
    FLASH_STATE_T state;

    state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->OBKEY = FLASH_KEY_1;
        FMC->OBKEY = FLASH_KEY_2;

        FMC->CTRL2_B.OBP = BIT_SET;

        *(__IO uint16_t*)addr = data;

        state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);
        
        if(state != FLASH_STATE_TIMEOUT)
        {
            FMC->CTRL2_B.OBP = BIT_RESET;
        }
    }

    return state;
}

/*!
 * @brief       Enable the specified page write protection
 *
 * @param       page:   Specifies the address of the pages to be write protected
 *                      This parameter can be any combination of the flowing values:
 *                      @arg FLASH_WRP_PAGE_0
 *                      @arg FLASH_WRP_PAGE_1
 *                      @arg FLASH_WRP_PAGE_2
 *                      @arg FLASH_WRP_PAGE_3
 *                      @arg FLASH_WRP_PAGE_4
 *                      @arg FLASH_WRP_PAGE_5
 *                      @arg FLASH_WRP_PAGE_6
 *                      @arg FLASH_WRP_PAGE_7
 *                      @arg FLASH_WRP_PAGE_8
 *                      @arg FLASH_WRP_PAGE_9
 *                      @arg FLASH_WRP_PAGE_10
 *                      @arg FLASH_WRP_PAGE_11
 *                      @arg FLASH_WRP_PAGE_12
 *                      @arg FLASH_WRP_PAGE_13
 *                      @arg FLASH_WRP_PAGE_14
 *                      @arg FLASH_WRP_PAGE_15
 *                      @arg FLASH_WRP_PAGE_16
 *                      @arg FLASH_WRP_PAGE_17 
 *                      @arg FLASH_WRP_PAGE_18
 *                      @arg FLASH_WRP_PAGE_19
 *                      @arg FLASH_WRP_PAGE_20
 *                      @arg FLASH_WRP_PAGE_21
 *                      @arg FLASH_WRP_PAGE_22
 *                      @arg FLASH_WRP_PAGE_23
 *                      @arg FLASH_WRP_PAGE_24
 *                      @arg FLASH_WRP_PAGE_25
 *                      @arg FLASH_WRP_PAGE_26 
 *                      @arg FLASH_WRP_PAGE_27
 *                      @arg FLASH_WRP_PAGE_28
 *                      @arg FLASH_WRP_PAGE_29
 *                      @arg FLASH_WRP_PAGE_30  
 *                      @arg FLASH_WRP_PAGE_31 
 *                      @arg FLASH_WRP_PAGE_ALL
 *
 * @retval      Returns the FLASH state.It can be combination of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_EnableWriteProtection(uint32_t page)
{
    uint8_t i;
    uint16_t temp;
    __IO uint16_t *WRPT; 
    FLASH_STATE_T state;

    WRPT = &OB->WRTPROT0;
    
    state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->OBKEY = FLASH_KEY_1;
        FMC->OBKEY = FLASH_KEY_1;
        
        FMC->CTRL2_B.OBP = BIT_SET;

        for(i = 0; i < 4; i++)
        {       
            temp = (uint16_t)~(page & 0xff);
            
            if((temp != 0xff) && (state == FLASH_STATE_COMPLETE))
            {
                WRPT[i] = temp;

                state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);
            }

            page >>= 8;
        }

        if(state != FLASH_STATE_TIMEOUT)
        {
            FMC->CTRL2_B.OBP = BIT_RESET;
        }
    }

    return state;
}

/*!
 * @brief       Read out protection configuration.
 *
 * @param       state:  Specifies the state.
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_ConfigReadOutProtection(FLASH_RDP_T rdp)
{
    FLASH_STATE_T state;

    state = FLASH_WaitForReady(FLASH_DELAY_ERASE);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->OBKEY = FLASH_KEY_1;
        FMC->OBKEY = FLASH_KEY_2;

        FMC->CTRL2_B.OBE = BIT_SET;
        FMC->CTRL2_B.STA = BIT_SET;

        state = FLASH_WaitForReady(FLASH_DELAY_ERASE);

        if(state == FLASH_STATE_COMPLETE)
        {
            FMC->CTRL2_B.OBE = BIT_RESET;

            FMC->CTRL2_B.OBP = BIT_SET;

            OB->READPROT = rdp;

            state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

            if(state != FLASH_STATE_TIMEOUT)
            {
                FMC->CTRL2_B.OBP = BIT_RESET;
            }
        }
        else if(state != FLASH_STATE_TIMEOUT)
        {
            FMC->CTRL2_B.OBE = BIT_SET;
        }
    }

    return state;
}

/*!
 * @brief       User option byte configuration
 *
 * @param       userConfig: Pointer to a FLASH_UserConfig_T structure that
 *                          contains the configuration information for User option byte
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_ConfigOptionByteUser(FLASH_UserConfig_T *userConfig)
{
    FLASH_STATE_T state;
    uint16_t temp;
    
    state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->OBKEY = FLASH_KEY_1;
        FMC->OBKEY = FLASH_KEY_2;

        FMC->CTRL2_B.OBP = BIT_SET;

        temp = userConfig->iwdtSw | userConfig->LIRCen | userConfig->trimHIRC | 
               userConfig->wwdtHalt | userConfig->wwdtSw | 0xe0;
               
        OB->USER = temp;

        state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);
        
        if(state != FLASH_STATE_TIMEOUT) 
        {
            FMC->CTRL2_B.OBP = BIT_RESET;
        }
    }

    return state;
}

/*!
 * @brief       AFR Option byte configuration
 *
 * @param       AFRConfig:  Pointer to a FLASH_AFRConfig_T structure that
 *                          contains the configuration information for AFR Option byte
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_ConfigOptionByteAFR(FLASH_AFRConfig_T *AFRConfig)
{
    FLASH_STATE_T state;
    uint16_t temp = 0;

    if(AFRConfig->AFR0 == FLASH_OB_AFR_DISABLE)
    {
        temp |= BIT0;
    }

    if(AFRConfig->AFR1 == FLASH_OB_AFR_DISABLE)
    {
        temp |= BIT1;
    }
    if(AFRConfig->AFR3 == FLASH_OB_AFR_DISABLE)
    {
        temp |= BIT3;
    }

    if(AFRConfig->AFR4 == FLASH_OB_AFR_DISABLE)
    {
        temp |= BIT4;
    }    
    if(AFRConfig->AFR7 == FLASH_OB_AFR_DISABLE)
    {
        temp |= BIT7;
    }     

    temp |= 0x64;
    
    state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->OBKEY = FLASH_KEY_1;
        FMC->OBKEY = FLASH_KEY_2;

        FMC->CTRL2_B.OBP = BIT_SET;

        OB->AFR = temp;

        state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);
        if(state != FLASH_STATE_TIMEOUT) 
        {
            FMC->CTRL2_B.OBP = BIT_RESET;
        }
    }

    return state;
}

/*!
 * @brief       Clock Option byte configuration
 *
 * @param       extClk:     Specifies the external clock selection
 *
 * @param       wuptClk:    Specifies the Wakeup timer clock source
 *
 * @param       wuptDiv:    Specifies the Wakeup timer divider
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_ConfigOptionByteClk(FLASH_OB_EXTCLK_T extClk, FLASH_OB_WUPT_CLKSRC_T wuptClk, 
                                                    FLASH_OB_WUPT_DIV_T wuptDiv)
{
    FLASH_STATE_T state;
    uint16_t temp;
    
    state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->OBKEY = FLASH_KEY_1;
        FMC->OBKEY = FLASH_KEY_2;

        FMC->CTRL2_B.OBP = BIT_SET;

        temp = extClk | wuptClk | wuptDiv | 0xf0;
        
        OB->CLKOPT = temp;

        state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);
        
        if(state != FLASH_STATE_TIMEOUT) 
        {
            FMC->CTRL2_B.OBP = BIT_RESET;
        }
    }

    return state;    
}

/*!
 * @brief       HXTCNT Option byte configuration
 *
 * @param       counter:    Specifies the stabilization time
 *
 * @retval      Returns the FLASH state.It can be one of value @ref FLASH_STATE_T
 *
 * @note       
 */
FLASH_STATE_T FLASH_ConfigOptionByteHXTCNT(FLASH_OB_HXTCNT_T counter)
{
    FLASH_STATE_T state;

    state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);

    if(state == FLASH_STATE_COMPLETE)
    {
        FMC->OBKEY = FLASH_KEY_1;
        FMC->OBKEY = FLASH_KEY_2;

        FMC->CTRL2_B.OBP = BIT_SET;
        
        OB->HXTCNT = counter;

        state = FLASH_WaitForReady(FLASH_DELAY_PROGRAM);
        
        if(state != FLASH_STATE_TIMEOUT) 
        {
            FMC->CTRL2_B.OBP = BIT_RESET;
        }
    }

    return state;
}

/*!
 * @brief       Read the FLASH User Option Bytes values
 *
 * @param       None
 *
 * @retval      The FLASH User Option Bytes value
 *
 * @note       
 */
uint32_t FLASH_ReadOptionByteUser(void)
{
    return (uint32_t)(FMC->OBCS >> 2);
}

/*!
 * @brief       Read the FLASH Write Protection Option Bytes Register value
 *
 * @param       None
 *
 * @retval      The FLASH Write Protection Option Bytes Register value
 *
 * @note       
 */
uint32_t FLASH_ReadOptionByteWriteProtection(void)
{
    return (uint32_t)(FMC->WRTPROT);
}

/*!
 * @brief       Checks whether the FLASH Read Protection Status is set or not
 *
 * @param       None
 *
 * @retval      FLASH ReadOut Protection Status(SET or RESET)
 *
 * @note       
 */
uint8_t FLASH_GetReadProtectionStatus(void)
{
    if(FMC->OBCS_B.READPROT)
    {
        return SET;
    }

    return RESET;
}

/*!
 * @brief       Checks whether the FLASH Prefetch Buffer status is set or not
 *
 * @param       None
 *
 * @retval      FLASH Prefetch Buffer Status (SET or RESET)
 *
 * @note       
 */
uint8_t FLASH_GetPrefetchBufferStatus(void)
{
    if(FMC->CTRL1_B.PBSF)
    {
        return SET;
    }

    return RESET;
}

/*!
 * @brief       Enable the specified FLASH interrupts
 *
 * @param       interrupt:  Specifies the FLASH interrupt sources
 *                          The parameter can be combination of following values:
 *                          @arg FLASH_INT_ERROR:       Error interruption
 *                          @arg FLASH_INT_COMPLETE:    operation complete interruption
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_EnableInterrupt(uint32_t interrupt)
{
    FMC->CTRL2 |= interrupt;
}

/*!
 * @brief       Disable the specified FLASH interrupts
 *
 * @param       interrupt:  Specifies the FLASH interrupt sources
 *                          The parameter can be combination of following values:
 *                          @arg FLASH_INT_ERROR:       Error interruption
 *                          @arg FLASH_INT_COMPLETE:    operation complete interruption
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_DisableInterrupt(uint32_t interrupt)
{
    FMC->CTRL2 &= ~interrupt;
}

/*!
 * @brief       Checks whether the specified FLASH flag is set or not
 *
 * @param       flag:   Specifies the FLASH flag to check
 *                      This parameter can be one of the values @ref FLASH_FLAG_T
 * @retval      None
 *
 * @note       
 */
uint8_t FLASH_ReadFlag(FLASH_FLAG_T flag)
{
    uint8_t status;
    
    if(flag & 0xff)
    {   
        status = FMC->STS & flag;
    }
    else
    {
        status = FMC->OBCS & flag;
    }

    if(status)
    {
        return SET;
    }

    return RESET;
}


/*!
 * @brief       Clear the specified FLASH flag
 *
 * @param       flag:   Specifies the FLASH flag to clear
 *              This parameter can be any combination of the following values:
 *              @arg FLASH_FLAG_BUSY:   Busy flag
 *              @arg FLASH_FLAG_PE:     Program error flag
 *              @arg FLASH_FLAG_WPE:    Write protection error flag
 *              @arg FLASH_FLAG_OC:     Operation complete flag
 *
 * @retval      None
 *
 * @note       
 */
void FLASH_ClearFlag(FLASH_FLAG_T flag)
{
    if(flag & 0xff)
    {
        FMC->STS = flag;
    }
}

/**@} end of group FLASH_Fuctions*/
/**@} end of group FLASH_Driver */
/**@} end of group Peripherals_Library*/

